/*
 * 爱组搭，低代码组件化开发平台
 * ------------------------------------------
 * 受知识产权保护，请勿删除版权申明，开发平台不允许做非法网站，后果自负
 */
package com.aizuda.generator;

import com.baomidou.mybatisplus.generator.config.po.TableField;
import com.baomidou.mybatisplus.generator.config.po.TableInfo;
import com.baomidou.mybatisplus.generator.engine.AbstractTemplateEngine;
import com.baomidou.mybatisplus.generator.engine.VelocityTemplateEngine;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.type.JdbcType;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 爱组搭 http://aizuda.com
 * ----------------------------------------
 * 自定义模板引擎执行类
 *
 * @author 青苗
 * @since 2021-10-31
 */
public class CodeTemplateEngine extends VelocityTemplateEngine {

    @Override
    public AbstractTemplateEngine batchOutput() {
        this.getConfigBuilder().getTableInfoList().forEach(this::validationAnnotations);
        return super.batchOutput();
    }

    protected void validationAnnotations(TableInfo tableInfo) {
        List<TableField> fields = tableInfo.getFields();
        fields.forEach(tableField -> {
            TableField.MetaInfo metaInfo = tableField.getMetaInfo();
            final String lt = "\t";
            final String lnt = "\n\t";
            final JdbcType jdbcType = metaInfo.getJdbcType();
            StringBuilder annotations = new StringBuilder();
            boolean tabLine = false;
            if (!metaInfo.isNullable()) {
                annotations.append(lt);
                if (JdbcType.VARCHAR == jdbcType) {
                    annotations.append("@NotBlank(groups = Create.class)");
                    tableInfo.addImportPackages("jakarta.validation.constraints.NotBlank");
                } else {
                    annotations.append("@NotNull(groups = Create.class)");
                    tableInfo.addImportPackages("jakarta.validation.constraints.NotNull");
                }
                tabLine = true;
            }
            if (JdbcType.VARCHAR == jdbcType) {
                annotations.append(tabLine ? lnt : lt);
                annotations.append("@Size(max = ").append(metaInfo.getLength()).append(")");
                tableInfo.addImportPackages("jakarta.validation.constraints.Size");
            } else if (JdbcType.BIGINT == jdbcType
                    || JdbcType.INTEGER == jdbcType
                    || JdbcType.TINYINT == jdbcType
                    || JdbcType.SMALLINT == jdbcType
                    || JdbcType.BIT == jdbcType
                    || JdbcType.FLOAT == jdbcType
                    || JdbcType.DOUBLE == jdbcType
                    || JdbcType.DECIMAL == jdbcType) {
                annotations.append(tabLine ? lnt : lt);
                annotations.append("@PositiveOrZero");
                tableInfo.addImportPackages("jakarta.validation.constraints.PositiveOrZero");
            }
            Map<String, Object> customMap = tableField.getCustomMap();
            if (MapUtils.isEmpty(customMap)) {
                customMap = new HashMap<>();
            }
            String va = annotations.toString();
            if (StringUtils.isNotBlank(va)) {
                va += lnt;
                tableInfo.addImportPackages("com.aizuda.core.validation.Create");
            } else {
                va = lt;
            }
            customMap.put("validationAnnotations", va);
            tableField.setCustomMap(customMap);
        });
    }
}
